from unittest import TestCase

from drf_spectacular.generators import SchemaGenerator
from drf_spectacular.plumbing import ComponentRegistry, ResolvedComponent
from rest_framework.views import APIView

from arkindex.project.mixins import DeprecatedMixin
from arkindex.project.openapi import AutoSchema


class TestAutoSchema(TestCase):

    def test_get_tags(self):
        """
        Test AutoSchema does not produce any tags automatically
        (must be manually defined on each endpoint)
        """
        self.assertListEqual(AutoSchema().get_tags(), [])

    def test_deprecated_response(self):
        """
        Test the DeprecatedAutoSchema marks everything as deprecated
        """
        class ThingView(DeprecatedMixin, APIView):

            def get(self, *args, **kwargs):
                pass

        registry = ComponentRegistry()
        view = SchemaGenerator().create_view(ThingView.as_view(), 'GET', None)
        operation = view.schema.get_operation(
            path='/test/',
            path_regex='/test/',
            path_prefix='/',
            method='GET',
            registry=registry,
        )

        self.assertDictEqual(
            operation,
            {
                'operationId': 'listThings',
                'deprecated': True,
                'description': '',
                'responses': {
                    '410': {
                        'content': {
                            'application/json': {
                                'schema': {'$ref': '#/components/schemas/DeprecatedException'}
                            }
                        },
                        'description': '',
                    }
                },
                'security': [
                    {'cookieAuth': []},
                    {'tokenAuth': []},
                    {'agentAuth': []}
                ],
            }
        )
        self.assertDictEqual(registry[('DeprecatedException', ResolvedComponent.SCHEMA)].schema, {
            'type': 'object',
            'required': ['detail'],
            'properties': {
                'detail': {
                    'type': 'string',
                },
            },
        })
